# Copyright (C) 2000-2003 Aver Development Corporation / Purple Magma Corporation. All Rights Reserved.

from __future__ import division


# There is a recurring desire to use weakrefs for providers. This does not
# work because often the only references to the providers are within the
# broker and hence would be garbage collected immediately.

class Broker(object):
	__slots__ = ()

	debug = False
	providers = {}
	currentTitles = []
	currentData = []


	def EnableDebugging(self, enable=True):
		Broker.debug = enable


	def __contains__(self, key):
		return key in self.providers


	def __delitem__(self, key):
		if Broker.debug:
			print r'Removing provider for "%s"' % key
		del Broker.providers[key]


	def __getitem__(self, key):
		return self.Request(key)


	def __setitem__(self, key, value):
		if Broker.debug:
			print r'Registering provider for "%s"' % key

		assert key not in Broker.providers

		Broker.providers[key] = value


	def Request(self, title, data={}):
		Broker.currentTitles.append(title)
		Broker.currentData.append(data)

		if Broker.debug:
			print r'Requesting "%s"' % title

		result = Broker.providers.get(title)()

		Broker.currentTitles.pop()
		Broker.currentData.pop()

		return result


	def CurrentTitle(self):
		if Broker.currentTitles:
			return Broker.currentTitles[-1]
		else:
			return None


	def CurrentData(self):
		if Broker.currentData:
			return Broker.currentData[-1]
		else:
			return None
