# Copyright (C) 2001-2003 Aver Development Corporation. All Rights Reserved.

from __future__ import division

import magma


class CalculatorEngine:
	transitions = {
				('# Entry', 'Numeric') :
					('self.AppendToX(key)', '# Entry'),

				('# Entry', 'Enter') :
					('self.Push(x)', 'Enter Pushed'),

				('# Entry', 'Arithmetic') :
					('self.Pop(2); self.Push(str(eval(y+key+x)))', 'Arithmetic Performed'),

				('Enter Pushed', 'Numeric') :
					('self.Pop(1); self.Push("0"); self.AppendToX(key)', '# Entry'),

				('Enter Pushed', 'Enter') :
					('self.Push(x)', 'Enter Pushed'),

				('Enter Pushed', 'Arithmetic') :
					('self.Pop(2); self.Push(str(eval(y+key+x)))', 'Arithmetic Performed'),

				('Arithmetic Performed', 'Numeric') :
					('self.Push("0"); self.AppendToX(key)', '# Entry'),

				('Arithmetic Performed', 'Enter') :
					('self.Push(x)', 'Arithmetic Performed'),

				('Arithmetic Performed', 'Arithmetic') :
					('self.Pop(2); self.Push(str(eval(y+key+x)))', 'Arithmetic Performed'),
			}

	def __init__(self):
		self.stack = ['0']
		magma.broker['Top of Stack'] = lambda: self.stack[-1]

		self.currentState = '# Entry'

		keyNames = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
					'decimal', 'div', 'mul', 'sub', 'add', 'Enter']
		for keyName in keyNames:
			magma.broadcaster.Register('Button ' + keyName,
										magma.curry(self.ButtonPressed, keyName))


	def ButtonPressed(self, key):
		if len(self.stack) < 1:
			x = '0'
		else:
			x = self.stack[-1]
		if len(self.stack) < 2:
			y = '0'
		else:
			y = self.stack[-2]

		if key == 'Enter':
			keyType = 'Enter'
		elif key in '0123456789':
			keyType = 'Numeric'
		elif key == 'decimal':
			keyType = 'Numeric'
			key = '.'
		else:
			keyType = 'Arithmetic'
			keyMap = {
						'div' : '/',
						'mul' : '*',
						'sub' : '-',
						'add' : '+',
					}
			key = keyMap[key]

		action, self.currentState = self.transitions[(self.currentState, keyType)]
		exec action

		magma.broadcaster.Broadcast('Update Display')
		magma.broadcaster.Broadcast('Set Status Bar Text', Text='Current State: ' + self.currentState)


	def AppendToX(self, key):
		x = self.stack.pop()
		if key != '.' or not '.' in x:
			x += key
			if len(x) == 2 and x[0] == '0' and x[1] != '.':
				x = x[1:]

		self.Push(x)


	def Pop(self, n):
		for i in range(n):
			if self.stack:
				self.stack.pop()


	def Push(self, x):
		self.stack.append(x)


engine = CalculatorEngine()
